' --------------------------------------------------------------------------------------
'
' QCard32Wrapper.VB                                               Ulrich Mller 19.09.02
'                                                                 PCS1700   VB.NET Final
'
' QCard32 KartenPool auf Basis der QCard32.DLL von 
' Stephen Murphy www.telusplanet.net/public/stevenm
' Der Kartenpool umfat zwei Spiele a 52 Karten, die von 1 - 104 durchnummeriert sind
' Zu jeder Karte sind statische Werte (Wert, Farbe ...) und dynamische Werte 
' (aktuelle Position, FaceUp, Blocked, Disabled ...) verfgbar. 
' Hinzukommen Methoden zur Verwaltung des Kartenpools und zur Handhabung einzelner
' Karten oder auch von Kartenfolgen (BlockDrag)
' Untersttzt wird das Positionieren und Zeichnen der Karten auf einer angebbaren
' (hwnd) Zeichenflche, beim Positionieren wird die aktuelle Position nachgehalten.
'
' --------------------------------------------------------------------------------------

Public Enum SuitCode As Integer
  Karo = 2  ' --- Diamonds
  Herz = 3  ' --- Hearts
  Pik = 4   ' --- Spades
  Kreuz = 1 ' --- Clubs
End Enum

Public Enum ColorCode As Integer
  Schwarz = 1
  Rot = 2
End Enum

Public Enum ValueCode As Integer
  Alle = 0
  Ass = 1
  Zwei = 2
  Drei = 3
  Vier = 4
  Fuenf = 5
  Sechs = 6
  Sieben = 7
  Acht = 8
  Neun = 9
  Zehn = 10
  Bube = 11
  Dame = 12
  Koenig = 13
End Enum

Public Class QCard32Exception
  Inherits Exception
  Public Sub New(ByVal message As String)
    MyBase.New(message)
  End Sub
End Class

' QCard32Pool -----------------------------------------------------------------------
' 
' Wrappen der QCard32.DLL - Funktionen
' 
' --------------------------------------------------------------------------------------

Public Class QCard32Pool


  Private PaintHandle As IntPtr

  ' --- Private Deklarationen ------------------------------------------------

  Private Declare Function InitializeDeckQC Lib "qcard32.dll" Alias "InitializeDeck" _
    (ByVal hwnd As IntPtr) As Integer
  Private Declare Sub DrawSymbolQC Lib "qcard32.dll" Alias "DrawSymbol" _
    (ByVal hwnd As IntPtr, ByVal nValue%, ByVal x%, ByVal y%)
  Private Declare Sub DrawCardQC Lib "qcard32.dll" Alias "DrawCard" _
    (ByVal hwnd As IntPtr, ByVal nCard%, ByVal x%, ByVal y%)
  Private Declare Sub DrawBackQC Lib "qcard32.dll" Alias "DrawBack" _
    (ByVal hwnd As IntPtr, ByVal nValue%, ByVal x%, ByVal y%)
  Private Declare Sub DealCardQC Lib "qcard32.dll" Alias "DealCard" _
    (ByVal hwnd As IntPtr, ByVal nCard%, ByVal x%, ByVal y%)
  Private Declare Sub RemoveCardQC Lib "qcard32.dll" Alias "RemoveCard" _
    (ByVal hwnd As IntPtr, ByVal nCard%)
  Private Declare Function InitDragQC Lib "qcard32.dll" Alias "InitDrag" _
    (ByVal hwnd As IntPtr, ByVal x%, ByVal y%) As Integer
  Private Declare Sub DoDragQC Lib "qcard32.dll" Alias "DoDrag" _
    (ByVal hwnd As IntPtr, ByVal x%, ByVal y%)
  Private Declare Function EndDragQC Lib "qcard32.dll" Alias "EndDrag" _
    (ByVal hwnd As IntPtr, ByVal x%, ByVal y%) As Integer
  Private Declare Sub ReturnDragQC Lib "qcard32.dll" Alias "ReturnDrag" _
    (ByVal hwnd As IntPtr, ByVal nCard%, ByVal nOldX%, ByVal nOldY%)
  Private Declare Sub BlockDragQC Lib "qcard32.dll" Alias "BlockDrag" _
    (ByVal hwnd As IntPtr, ByRef nFirst%, ByVal nNumCards%, ByVal x%, ByVal y%)
  Private Declare Function EndBlockDragQC Lib "qcard32.dll" Alias "EndBlockDrag" _
    (ByVal hwnd As IntPtr, ByRef nFirst%, ByVal nNumCards%, ByVal x%, ByVal y%) As Integer
  Private Declare Sub ReturnBlockDragQC Lib "qcard32.dll" Alias "ReturnBlockDrag" _
    (ByVal hwnd As IntPtr, ByRef nFirst%, ByVal nNumCards%, ByVal x%, ByVal y%)

  Private NrBack As Integer = 1

  ' --- Konstruktor ------------------------------------------------------------------

  Sub New(ByVal handle As IntPtr)
    If InitializeDeckQC(handle) = 0 Then _
        Throw New QCard32Exception("LadeProblem bei QCard32.DLL")
    PaintHandle = handle
  End Sub

  ' --- QCard32.DLL Eigenschaften und Konstanten -------------------------------------

  Public Const FaceDown = 0
  Public Const FaceUp = 1
  Public Const CardWidth = 71
  Public Const CardWidthS = CardWidth + 3
  Public Const CardHeight = 96
  Public Const CardHeightS = CardHeight + 3
  Public Const Offset = 20
  Public Const KOffset = 12

  Public Property BackNr() As Integer
    Get
      Return Me.NrBack
    End Get
    Set(ByVal Value As Integer)
      Me.NrBack = Value
    End Set
  End Property

  ' --- Reset all card values to defaults
  Public Declare Sub SetDefaultValues Lib "qcard32.dll" ()
  ' --- Set the currently used card back design for cards 105 to 109
  Public Declare Sub SetCurrentBack Lib "qcard32.dll" (ByVal nIndex%)

  ' --- Card drawing subs ---------------------

  Public Sub DrawSymbol(ByVal nValue%, ByVal x%, ByVal y%)
    DrawSymbolQC(PaintHandle, nValue, x, y)
  End Sub
  Public Sub DrawCard(ByVal nCard%, ByVal x%, ByVal y%)
    DrawCardQC(PaintHandle, nCard, x, y)
  End Sub
  Public Sub DrawBack(ByVal nValue%, ByVal x%, ByVal y%)
    DrawBackQC(PaintHandle, nValue, x, y)
  End Sub
  Public Sub DealCard(ByVal nCard%, ByVal x%, ByVal y%)
    DealCardQC(PaintHandle, nCard, x, y)
  End Sub
  Public Sub RemoveCard(ByVal nCard%)
    RemoveCardQC(PaintHandle, nCard)
  End Sub

  ' --- Get card information functions --------------

  Public Declare Function GetCardColor Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function GetCardSuit Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function GetCardValue Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function GetCardStatus Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function GetCardBlocked Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function IsCardDisabled Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function GetCardX Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function GetCardY Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function GetUser1 Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function GetUser2 Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function GetUser3 Lib "qcard32.dll" (ByVal nCard%) As Integer
  Public Declare Function GetUser4 Lib "qcard32.dll" (ByVal nCard%) As Integer

  ' --- Set card information subs ------------------------------

  Public Declare Sub SetCardStatus Lib "qcard32.dll" (ByVal nCard%, ByVal bValue%)
  Public Declare Sub AdjustCardBlocked Lib "qcard32.dll" (ByVal nCard%, ByVal bValue%)
  Public Declare Sub SetCardDisabled Lib "qcard32.dll" (ByVal nCard%, ByVal bValue%)
  Public Declare Sub SetCardX Lib "qcard32.dll" (ByVal nCard%, ByVal x%)
  Public Declare Sub SetCardY Lib "qcard32.dll" (ByVal nCard%, ByVal y%)
  Public Declare Sub SetUser1 Lib "qcard32.dll" (ByVal nCard%, ByVal bValue%)
  Public Declare Sub SetUser2 Lib "qcard32.dll" (ByVal nCard%, ByVal nValue%)
  Public Declare Sub SetUser3 Lib "qcard32.dll" (ByVal nCard%, ByVal nValue%)
  Public Declare Sub SetUser4 Lib "qcard32.dll" (ByVal nCard%, ByVal nValue%)
  Public Declare Sub SetOffSet Lib "qcard32.dll" (ByVal nValue%)

  ' --- Dragging subs and functions --------------------------------

  Public Function InitDrag(ByVal x%, ByVal y%) As Integer
    Return InitDragQC(PaintHandle, x, y)
  End Function
  Public Declare Sub AbortDrag Lib "qcard32.dll" ()
  Public Sub DoDrag(ByVal x%, ByVal y%)
    DoDragQC(PaintHandle, x, y)
  End Sub
  Public Function EndDrag(ByVal x%, ByVal y%) As Integer
    Return EndDragQC(PaintHandle, x, y)
  End Function
  Public Sub ReturnDrag(ByVal nCard%, ByVal nOldX%, ByVal nOldY%)
    ReturnDragQC(PaintHandle, nCard, nOldX, nOldY)
  End Sub

  Public Sub BlockDrag(ByRef nFirst%, ByVal nNumCards%, ByVal x%, ByVal y%)
    BlockDragQC(PaintHandle, nFirst, nNumCards, x, y)
  End Sub
  Public Function EndBlockDrag(ByRef nFirst%, ByVal nNumCards%, _
                                                        ByVal x%, ByVal y%) As Integer
    Return EndBlockDragQC(PaintHandle, nFirst, nNumCards, x, y)
  End Function
  Public Sub ReturnBlockDrag(ByRef nFirst%, ByVal nNumCards%, ByVal x%, ByVal y%)
    ReturnBlockDragQC(PaintHandle, nFirst, nNumCards, x, y)
  End Sub

  ' --- Undocumented functions, generally not used ------------------------

  Public Declare Function GetFreeDestination _
                          Lib "qcard32.dll" (ByVal nSource%) As Integer
  ' --- returns the number of any unblocked card which lies 
  '     beneath the mouse coordinates x, y
  Private Declare Function PointInFreeCard Lib "qcard32.dll" _
    (ByVal x%, ByVal y%) As Integer

  ' --- returns the number of any card whose top 16 (or OffSet) pixels lie 
  '     beneath the mouse coordinates x, y
  Private Declare Function PointInCardTop _
                           Lib "qcard32.dll" (ByVal x%, ByVal y%) As Integer

  ' --- manually sets the active drag card for a subsequent DoDrag or BlockDrag call
  Private Declare Sub SetActiveCard Lib "qcard32.dll" (ByVal nCard%)

End Class
