' --------------------------------------------------------------------------------------
'
' Talon.VB                                                        Ulrich Mller 19.09.02
'                                                                 PCS1700   VB.NET Final
'
' Klassen zur Erstellung von Patiencen auf Basis von QCard32.DLL (QCard32Wrapper)
' Klassen fr Kartenpaket und Talon
'
' --------------------------------------------------------------------------------------

' KartenPaket --------------------------------------------------------------------------
'            Talon mit ein oder zwei Spielen. Karten liegen FaceDown direkt bereinander
' --------------------------------------------------------------------------------------
Public Class KartenPaket

  Private qc As QCard32Pool
  Private FID%
  Private actCard%, anzCards%
  Private Cards() As Card
  Private XBasis, YBasis As Integer

  ' New -------------------------------------------------------------------------------
  '     Konstruktor
  ' -----------------------------------------------------------------------------------
  Sub New(ByVal QCard As QCard32Pool, ByVal NrPacks%, ByVal Left%, ByVal Top%)
    FID = 0
    qc = QCard
    actCard = -1
    anzCards = IIf(NrPacks = 1, 52, 104)
    ReDim Cards(anzCards - 1)
    XBasis = Left
    YBasis = Top
  End Sub

  ' Left / Top -------------------------------------------------------------------------
  '                 Lesen der Koordinaten der linken, oberen Ecke (Pixel)
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property Left() As Integer
    Get
      Return XBasis
    End Get
  End Property
  Public ReadOnly Property Top() As Integer
    Get
      Return YBasis
    End Get
  End Property

  ' Range ------------------------------------------------------------------------------
  '       Lesen der Bereichswerte von KartenPaket
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property Range() As Rectangle
    Get
      Return New Rectangle(XBasis, YBasis, qc.CardWidth, qc.CardHeight)
    End Get
  End Property

  ' ID ---------------------------------------------------------------------------------
  '    Lesen der ID des gemischten Kartenpakets (Doppelpack)
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property ID() As Integer
    Get
      Return FID
    End Get
  End Property

  ' CardCount ---------------------------------------------------------------------------
  '           Anzahl Karten im Talon
  ' -------------------------------------------------------------------------------------
  Public ReadOnly Property CardCount() As Integer
    Get
      Return actCard + 1
    End Get
  End Property

  ' TopMostCard ------------------------------------------------------------------------
  '             Lesen der obersten Karte des Talon
  ' ------------------------------------------------------------------------------------
  Public ReadOnly Property TopMostCard() As Card
    Get
      If actCard < 0 Then Return Nothing Else Return Cards(actCard)
    End Get
  End Property

  ' RemoveCard -------------------------------------------------------------------------
  '            Entfernen der obersten Karte des Talon
  ' ------------------------------------------------------------------------------------
  Public Function RemoveCard(Optional ByVal FaceUp As Boolean = True) As Card
    If actCard < 0 Then Return Nothing
    If FaceUp Then Cards(actCard).Face = True
    qc.RemoveCard(Cards(actCard).CardNr)
    actCard -= 1
    Return Cards(actCard + 1)
  End Function

  ' AddCard ----------------------------------------------------------------------------
  '         Srcard auf dem KartenPaket ablegen
  ' ------------------------------------------------------------------------------------
  Public Sub AddCard(ByVal SrcCard As Card)
    actCard += 1
    Cards(actCard) = SrcCard
    SrcCard.PicX = XBasis
    SrcCard.PicY = YBasis
    SrcCard.Area = AreaCode.Talon
    SrcCard.PosC = 0
    SrcCard.PosR = actCard
    SrcCard.Face = False
    qc.DealCard(SrcCard.CardNr, XBasis, YBasis)
  End Sub

  ' Shuffle ---------------------------------------------------------------------------
  '         Mischen CardNr nach ID und fllen Cards mit Card 
  ' -----------------------------------------------------------------------------------
  Public Sub Shuffle(ByVal ID%)

    Dim i, j, ks, sw As Integer
    Dim CardNrs%(anzCards - 1)
    Dim r As New Random(ID)

    FID = ID
    For i = 0 To anzCards - 1            ' --- mit Kartennummern fllen
      CardNrs(i) = i + 1
    Next i
    For i = 1 To 10
      For j = 0 To anzCards - 1          ' --- 10mal alle Karten zuf. Tauschen
        ks = r.Next(0, anzCards - 1)
        sw = CardNrs(j)
        CardNrs(j) = CardNrs(ks)
        CardNrs(ks) = sw
      Next j
    Next i

    ' ------- Init KartenPool in QCard32.DLL ---------------------------------
    qc.SetDefaultValues()
    qc.SetOffSet(qc.Offset)
    qc.DrawSymbol(2, XBasis, YBasis)
    ' ------- Erstellen der Karten-Objekte in Cards -------------------------
    For i = 0 To anzCards - 1
      Cards(i) = New Card(qc, CardNrs(i))
      Cards(i).PicX = XBasis
      Cards(i).PicY = YBasis
      Cards(i).Area = AreaCode.Talon
      Cards(i).PosC = 0
      Cards(i).PosR = i
      Cards(i).Face = False
      qc.DealCard(Cards(i).CardNr, XBasis, YBasis)
    Next i
    actCard = anzCards - 1
  End Sub

  ' Paint ------------------------------------------------------------------------------
  '       Zeichnen des Kartenbildes (Kartenrckseite / Leersymbol)
  ' ------------------------------------------------------------------------------------
  Public Sub Paint()
    If actCard < 0 Then
      qc.DrawSymbol(2, XBasis, YBasis)
    Else
      qc.DrawBack(qc.BackNr, XBasis, YBasis)
    End If
  End Sub

End Class
