' --------------------------------------------------------------------------------------
'
' SolPatience.VB                                                  Ulrich Mller 16.09.02
'                                                                 PCS1700   VB.NET Final
' Solitr Patience : Logik, Legen und Anzeigen
' Basis            : PotiPat.DLL und Qcard32.DLL
'
' Ablage  : Ablage48, obere Reihe, 4 Pltze, von Ass in einer Farbe aufsteigend
' Auslage : AuslageV, Hauptfeld, 7 Leitern, absteigend in wechselnden Farben
' Keller  : LiFoStack, obere Reihe, links, bereinander
' Talon   : KartenPack, obere Reihe, ganz links, bereinander, 
'           Mehrfachdurchlauf von Keller
'
' --------------------------------------------------------------------------------------

Imports PotiPat

Public Class SolException
  Inherits Exception
  Public Sub New(ByVal message$)
    MyBase.New(message)
  End Sub
End Class

' SolPatience --------------------------------------------------------------------------
'             Patiencespezifische Methoden fr die Solitr Patience
' --------------------------------------------------------------------------------------
Public Class SolPatience
  Inherits PotiPatience
  Public Event GameWon(ByVal Pati As SolPatience)

  ' --------------------- Private Felder ----------------------------------------------
  Private Talon As KartenPaket
  Private Keller As LiFoStack
  Private Ablage As Ablage48
  Private Auslage As AuslageV
  Private SrcCard As Card, SrcPos As New CardPos()
  Private CardList(19), anzCard As Integer

  Private lblRest As Label

  ' New --------------------------------------------------------------------------------
  '     Konstruktor
  ' ------------------------------------------------------------------------------------
  Sub New(ByVal Canvas As Form, ByVal lblRest As Label)
    MyBase.New(Canvas)

    Me.lblRest = lblRest

    ' --- Spielbereichs Objekte ---------------------------------------------------------
    Talon = New KartenPaket(qc, 1, 16, 16)
    Keller = New LiFoStack(qc, 16 + qc.CardWidth + 12, 16)
    Ablage = New Ablage48(qc, 4, qc.CardWidth + 12, 16 + ((qc.CardWidth + 12) * 3), 16)
    Auslage = New AuslageV(qc, 7, qc.CardWidth + 12, 16, 16 + qc.CardHeightS, 500)

    ' ----- Anzeigen ausrichten ---------------------------------------------------------
    lblRest.Left = Keller.Left + Keller.Range.Width + _
                                    (qc.CardWidth + 12 + 12) / 2 - lblRest.Width / 2
    lblRest.Top = Keller.Top + qc.CardHeight / 2 - lblRest.Height / 2
  End Sub

  ' FindDestPos -----------------------------------------------------------------------
  '          Bestimmen der durch die Pixel-Koordinaten vorgegebenen Position
  ' -----------------------------------------------------------------------------------
  Private Function FindDestPos(ByVal X%, ByVal Y%) As CardPos

    Dim DestPos As New CardPos()

    DestPos.CardNr = 0
    If Ablage.Range.Contains(X, Y) Then
      DestPos.Area = AreaCode.Ablage
      DestPos.PosC = Ablage.XToCol(X)
    ElseIf Auslage.Range.Contains(X, Y) Then
      DestPos.Area = AreaCode.Auslage
      DestPos.PosC = Auslage.XToCol(X)
    ElseIf Keller.Range.Contains(X, Y) Then
      DestPos.Area = AreaCode.Keller
      DestPos.PosC = 0
    Else
      DestPos.Area = AreaCode.none
      DestPos.PosC = 0
    End If
    DestPos.PosR = -1
    Return DestPos
  End Function

  ' CardToAblage ----------------------------------------------------------------------
  '              Prfen und Ablegen Karte von Auslage oder Keller nach Ablage
  '              Solitarspezifisch --> nur Ablage Spalte 0 - 3
  ' -----------------------------------------------------------------------------------
  Private Function CardToAblage(ByVal SrcCard As Card) As Boolean
    Dim DestCol% = Choose(SrcCard.Suit, 0, 3, 2, 1)
    If Ablage.IsAscendingSuit(SrcCard, DestCol) Then
      If SrcCard.Area = AreaCode.Auslage Then
        Ablage.AddCard(Auslage.RemoveCard(SrcCard.PosC), DestCol)
      Else
        Ablage.AddCard(Keller.RemoveCard(), DestCol)
      End If
      Return True
    End If
    Return False
  End Function

  ' SpielAuslegen ---------------------------------------------------------------------
  '               Auslegen der Solitr Patience
  ' -----------------------------------------------------------------------------------
  Private Sub SpielAuslegen(ByVal Talon As KartenPaket)

    Dim i, j, OG As Integer
    Dim SrcCard As Card

    Auslage.Clear()
    OG = 0
    For i = 0 To 6
      For j = 0 To OG
        SrcCard = Talon.RemoveCard()
        If j = OG Then
          Auslage.AddCard(SrcCard, i, True)
        Else
          Auslage.AddCard(SrcCard, i, False)
          SrcCard.Disabled = True
        End If
      Next j
      OG += 1
    Next i
  End Sub

  ' SpielStarten -----------------------------------------------------------------------
  '              Starten eines neuen Spieles : Initwerte, Mischen, Auslegen
  ' ------------------------------------------------------------------------------------
  Public Overrides Sub SpielStarten(ByVal SpielNr%)
    Canvas.CreateGraphics.Clear(Color.Green)
    Talon.Shuffle(SpielNr)
    Keller.Clear()
    Ablage.Clear()
    SpielAuslegen(Talon)
    lblRest.Text = "52"
    lblRest.Visible = True
  End Sub

  ' Refresh ----------------------------------------------------------------------------
  '         Wiederherstellen des aktuellen Kartenbildes
  ' ------------------------------------------------------------------------------------
  Public Overrides Sub Refresh()
    Talon.Paint()
    Keller.Paint()
    Ablage.Paint()
    Auslage.Paint()
  End Sub

  ' StartZiehen ------------------------------------------------------------------------
  '             Starten des Ziehens einer Karte wenn Maus ber der Karte
  ' ------------------------------------------------------------------------------------
  Public Overrides Sub StartZiehen(ByVal X%, ByVal Y%)

    Dim SrcNr% = qc.InitDrag(X, Y)
    Dim i, j As Integer

    If SrcNr = 0 Then           ' --- ber leerem Talon -------------------------------
      SrcPos.CardNr = 0
      SrcPos.PicX = X
      SrcPos.PicY = Y
      qc.AbortDrag()
      DragMode = PotiPatience.DragCode.None
    Else                        ' ------------ BlockDrag -----------------------------
      If qc.GetCardBlocked(SrcNr) Then
        DragMode = DragCode.Block
        SrcCard = Auslage.ActualCard(qc.GetUser3(SrcNr), qc.GetUser4(SrcNr))
        j = 0       ' --- Kartenliste fr BlockDrag --------------------------
        For i = SrcCard.PosR To Auslage.CardCount(SrcCard.PosC) - 1
          CardList(j) = Auslage.ActualCard(SrcCard.PosC, i).CardNr
          j += 1
        Next i
        anzCard = Auslage.CardCount(SrcCard.PosC) - SrcCard.PosR
      Else                      ' -------------- Einzelkarte --------------------------
        DragMode = DragCode.One
        Select Case qc.GetUser2(SrcNr)
          Case AreaCode.Ablage
            SrcCard = Ablage.TopMostCard(qc.GetUser3(SrcNr))
          Case AreaCode.Auslage
            SrcCard = Auslage.TopMostCard(qc.GetUser3(SrcNr))
          Case AreaCode.Keller
            SrcCard = Keller.TopMostCard()
          Case AreaCode.Talon
            SrcCard = Talon.TopMostCard()
            qc.AbortDrag()
        End Select
      End If
    End If

  End Sub

  ' WeiterZiehen -----------------------------------------------------------------------
  '              Fortsetzen des Ziehens
  ' ------------------------------------------------------------------------------------
  Public Overrides Sub WeiterZiehen(ByVal X%, ByVal Y%)

    Select Case DragMode
      Case DragCode.Block
        qc.BlockDrag(CardList(0), anzCard, X, Y)
      Case DragCode.One
        qc.DoDrag(X, Y)
      Case Else
    End Select

  End Sub

  ' EndeZiehen -------------------------------------------------------------------------
  '            Beenden des Ziehens, Ablegen bzw. Rcklegen der Karte
  ' ------------------------------------------------------------------------------------
  Public Overrides Sub EndeZiehen(ByVal X%, ByVal Y%)

    Dim DestPos As CardPos

    Select Case DragMode

      Case DragCode.One           ' --- Einzelkarte --------------------------
        qc.EndDrag(X, Y)
        DestPos = FindDestPos(X, Y)

        Select Case DestPos.Area

          Case AreaCode.Ablage    ' --- in die Ablage ------------------------
            If Ablage.IsAscendingSuit(SrcCard, DestPos.PosC) Then
              If SrcCard.Area = AreaCode.Auslage Then
                Ablage.AddCard(Auslage.RemoveCard(SrcCard.PosC), DestPos.PosC)
              ElseIf SrcCard.Area = AreaCode.Keller Then
                Ablage.AddCard(Keller.RemoveCard(), DestPos.PosC)
              Else
                qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY)
              End If
            Else
              qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY)
            End If

          Case AreaCode.Auslage   ' --- in die Auslage ------------------------
            If Auslage.IsDescendingChanging(SrcCard, DestPos.PosC, ValueCode.Koenig) Then
              If SrcCard.Area = AreaCode.Auslage Then
                Auslage.AddCard(Auslage.RemoveCard(SrcCard.PosC), DestPos.PosC)
              ElseIf SrcCard.Area = AreaCode.Keller Then
                Auslage.AddCard(Keller.RemoveCard(), DestPos.PosC, True)
              Else
                qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY)
              End If
            Else
              qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY)
            End If

          Case AreaCode.Keller    ' --- in den Keller ---------------------------
            If SrcCard.Area = AreaCode.Talon Then
              Keller.AddCard(Talon.RemoveCard())
            Else
              qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY)
            End If
          Case Else
            qc.ReturnDrag(SrcCard.CardNr, SrcCard.PicX, SrcCard.PicY)
        End Select

      Case DragCode.Block     ' --- BlockDrag, Umlegen ohne Rcksicht auf freie Pltze
        qc.EndBlockDrag(CardList(0), anzCard, X, Y)
        DestPos = FindDestPos(X, Y)
        If DestPos.Area = AreaCode.Auslage And _
        Auslage.IsDescendingChanging(SrcCard, DestPos.PosC, ValueCode.Koenig) Then
          Auslage.AddRange(Auslage.RemoveRange(SrcCard.PosR, SrcCard.PosC), DestPos.PosC)
        Else
          qc.ReturnBlockDrag(CardList(0), anzCard, SrcCard.PicX, SrcCard.PicY)
        End If
      Case Else
        Return
    End Select

    ' ------------------ Beenden Drag ----------------------------------------

    DragMode = DragCode.None
    lblRest.Text = 52 - Ablage.CardCount
    If Ablage.CardCount = 52 Then RaiseEvent GameWon(Me)
  End Sub

  ' KarteAblegen ----------------------------------------------------------------------
  '              Prfen und Ablegen Karte
  ' ------------------------------------------------------------------------------------
  Public Overrides Sub KarteAblegen()

    Dim Karte As Card

    If DragMode <> DragCode.None Then   ' --- EinzelKarte ablegen
      qc.AbortDrag()
      DragMode = DragCode.None

      Select Case SrcCard.Area          ' --- Karte von Auslage, Keller auf Ablage
        Case AreaCode.Auslage, AreaCode.Keller
          CardToAblage(SrcCard)
        Case AreaCode.Talon             ' --- Karte von Talon auf Keller
          Keller.AddCard(Talon.RemoveCard())
        Case Else
      End Select

    Else  ' --- Talon leer, Keller wenden und auf Talon legen ------------------------
      If Talon.Range.Contains(SrcPos.PicX, SrcPos.PicY) Then
        Karte = Talon.RemoveCard()
        Do Until IsNothing(Karte)
          Keller.AddCard(Karte)
          Karte = Talon.RemoveCard()
        Loop
        Karte = Keller.RemoveCard()
        Do Until IsNothing(Karte)
          Talon.AddCard(Karte)
          Karte = Keller.RemoveCard()
        Loop
      End If
    End If

    lblRest.Text = 52 - Ablage.CardCount
    If Ablage.CardCount = 52 Then RaiseEvent GameWon(Me)

  End Sub
End Class
